#!/usr/bin/env python

"""
$Id: gc_stat.py 5753 2019-11-22 12:10:32Z nicklas $

Script to create a VCF-like file with GC content and DNA strength INFO fields. This script is
part of a pipeline that requires a specially prepared bed-file as input (reading from stdin).
Columns (0-based for convenience):

0: Chromosome (printed to VCF#CHROM)
1: Start (=original location - 50; or 0 if original location < 50)
2: Stop (=original location + 50; not used)
3: Original location (printed to VCF#POS)
4: REF  (printed to VCF#REF)
5: ALT (printed to VCF#ALT)
6: SEQUENCE between start and stop (used for calculating GC and DNA strength, which are printed to VCF#INFO)

The VCF file that is produced must be compressed (bgzip) and indexed (tabix) before it can be used by vcfanno 
to annotate the orginal VCF file (from VarDict) with the GC content and DNA strength. A .toml file like 
this is needed:

[[annotation]]
file="/path/to/vcf-file-produced-by-this-script.gz"
fields=["GC_CONT","GC_CONT_LOCAL","STRENGTH","STRENGTH_LOCAL"]
names=["GC_CONT","GC_CONT_LOCAL","STRENGTH","STRENGTH_LOCAL"]
ops=["self","self","self","self"]

"""

import sys

NORM_LEN = 50
LOCAL_LEN = 6

# Column indexes from the BED file we are reading
COL_CHR = 0
COL_START = 1
COL_END = 2
COL_POS = 3
COL_REF = 4
COL_ALT = 5
COL_SEQ = 6

# GC content and DNA strength values
GC_CONT_VALUES = { "A": 0.0, "C": 1.0, "G": 1.0, "N": 0.5, "T": 0.0 }
DNA_STRENGTH_VALUES = {
    "AA": 5.0, "AC": 10.0, "AG": 8.0, "AN": 7.5, "AT": 7.0,
    "CA": 7.0, "CC": 11.0, "CG": 10.0, "CN": 9.0, "CT": 8.0, 
    "GA": 8.0, "GC": 13.0, "GG": 11.0, "GN": 10.5, "GT": 10.0,
    "NA": 6.0, "NC": 10.5, "NG": 9.0, "NN": 8.25, "NT": 7.5,
    "TA": 4.0, "TC": 8.0, "TG": 7.0, "TN": 6.0, "TT": 5.0
  }

def getSequenceInfo(seqIn):
    """
    Returns GC-content and DNA-strength of the seqIn
    """
    seqIn = seqIn.upper()
    gcContSum = 0
    strengthSum = 0
    for seqPos in range(0, len(seqIn)):
        if seqPos > 1:
            strengthSum += DNA_STRENGTH_VALUES[seqIn[seqPos - 2:seqPos]]
        gcContSum += GC_CONT_VALUES[seqIn[seqPos]]

    gcCont = gcContSum / len(seqIn)
    dnaStrength = strengthSum / len(seqIn)
    return gcCont, dnaStrength


header = '''##fileformat=VCFv4.2
##INFO=<ID=GC_CONT,Number=1,Type=Float,Description="Proportion of bases that are GC within {norm} bases up- and down-stream of position">
##INFO=<ID=GC_CONT_LOCAL,Number=1,Type=Float,Description="Proportion of bases that are GC within {loc} bases up and downstream of position">
##INFO=<ID=STRENGTH,Number=1,Type=Float,Description="DNA strength in the region within {norm} bases up- and down-stream of position">
##INFO=<ID=STRENGTH_LOCAL,Number=1,Type=Float,Description="DNA strength in the region within {loc} bases up- and down-stream of position">
#CHROM  POS     ID      REF     ALT     QUAL    FILTER  INFO'''.format(norm=NORM_LEN, loc=LOCAL_LEN)

print(header)

for line in sys.stdin:
    cols = line.rstrip().split('\t')
    seq = cols[COL_SEQ]
    posInSeq = int(cols[COL_POS])-int(cols[COL_START])
    seqLocal = cols[COL_SEQ][max(posInSeq-LOCAL_LEN, 0):posInSeq+LOCAL_LEN]
    gcNorm, strengthNorm = getSequenceInfo(seq)
    gcLocal, strengthLocal = getSequenceInfo(seqLocal)
    cout = (cols[COL_CHR], cols[COL_POS], '.', cols[COL_REF], cols[COL_ALT], '.', '.', gcNorm, gcLocal, strengthNorm, strengthLocal)
    print('{0}\t{1}\t{2}\t{3}\t{4}\t{5}\t{6}\tGC_CONT={7};GC_CONT_LOCAL={8};STRENGTH={9};STRENGTH_LOCAL={10}'.format(*cout))
