#!/bin/bash
##
## Pipeline script for the mBaf analysis. It requries an 'alignment.bam' file and
## output genotypes in 'mbaf_genotype.vcf' to the same directory.
##
## Environment variables that should be defined (in job.sh) before calling this script
##  -BamFolder: Path to folder where alignment.bam is found (results will also be saved here)
##  -BamName: Base part of alignment BAM file (eg. alignment)
##  -JAVA: Path to JAVA program
##  -GATK: Path to GATK JAR file
##  -DBSNP: Path to VCF file with SNP variants that should be genotyped
##  -HCRef: Path to reference genome FASTA file
##  -HaplotypeCallerOptions: Other options for the the HaplotypeCaller program
##  -TMPDIR: Path to a local temporary working directory
##

set -e

## Import utility functions
source ./reggie-utils.sh

## Verify settings in options
rg_var_isdir "BamFolder" "TMPDIR" "WD"
rg_var_isfile "JAVA" "GATK" "DBSNP" "HCRef"
rg_var_isset "BamName"
rg_file_exists "${BamFolder}/${BamName}.bam" "${BamFolder}/${BamName}.bai"

## Move to the temporary working directory
cd ${TMPDIR}
mkdir -p bam
mkdir -p mbaf
mkdir -p done

## Copy BAM file to local working directory
if [ ! -f "done/copy.done" ]; then
	rg_progress 10 "Copying BAM file"
	cp ${BamFolder}/${BamName}.bam bam/alignment.bam
	cp ${BamFolder}/${BamName}.bai bam/alignment.bai
	touch "done/copy.done"
fi

## Run GATK HaplotypeCaller
if [ ! -f "done/haplotypecaller.done" ]; then
	rg_progress 20 "Running HaplotypeCaller"
	${WD}/stdwrap.sh ${JAVA} ${JavaOptions} \
		-jar ${GATK} -T HaplotypeCaller \
		-R ${HCRef} \
		--genotyping_mode GENOTYPE_GIVEN_ALLELES \
		-L ${DBSNP} --dbsnp ${DBSNP} --alleles ${DBSNP} \
		${HaplotypeCallerOptions} \
		-I bam/alignment.bam \
		-o mbaf/mbaf_genotype.vcf \
		> mbaf/mbaf_HaplotypeCaller.out
	touch "done/haplotypecaller.done"
fi

rg_progress 90 "Copying result files to project archive"
\cp mbaf/mbaf* ${BamFolder}
ls -1 mbaf/mbaf* > ${WD}/files.out

rg_progress 99 "Analysis completed, cleaning up..."
