#!/bin/bash
##
## Pipeline script for the RNAseq demux step. 
## 
##
## Environment variables that should be defined (in job.sh) before calling this script
##  -AllRunArchives: White-space separated list of locations to search for sequencing data
##  -JAVA: Path to Java
##  -PICARD: Path to Picard JAR file
##  -PICARD_MEMORY: Amount of memory to assign to Java when running Picard
##  -BOWTIE: Path to BowTie program
##  -BowTieOptions: Other options for the BowTie program
##  -Gidx: Path to reference genome used for alignment with BowTie
##  -TRIMMOMATIC: Path to Trimmomatich JAR file
##  -TrimmomaticOptions1: Options for the first Trimmomatic step (adapter filter)
##  -TrimmomaticOptions2: Options for the second Trimmomatic step (quality filter)

set -e

## Import utility functions
source ./reggie-utils.sh
source ./demux-utils.sh

## Verify settings in options
rg_var_isdir "TMPDIR" "WD"
rg_var_isfile "JAVA" "PICARD" "BOWTIE" "TRIMMOMATIC" 
rg_var_isset "AllRunArchives" "BowTieOptions" "Gidx" "TrimmomaticAdapterOptions" "TrimmomaticQualityOptions"


## Move to the temporary working directory and create subdirectories
cd ${TMPDIR}
mkdir -p done
mkdir -p fastq
mkdir -p fastq.merged
mkdir -p fastq.aligned
mkdir -p fastq.trim.1
mkdir -p fastq.trim.2

## Read information about each flowcell and lane from 'lane_info.txt' and run picard to create FASTQ files
## Columns in 'lane_info.txt': 
##  -FLOWCELL_BARCODE
##  -LANE
##  -DEMUX_NAME
##  -POOL_NAME
##  -DATA_FILES_FOLDER
##  -READSTRING
##  -EXTRACT_OPTIONS
##  -FASTQ_OPTIONS
NUM_LINES=`wc -l < ${WD}/lane_info.txt`
CURRENT_LINE=0
while IFS=$',' read FLOWCELL_BARCODE LANE DEMUX_NAME POOL_NAME DATA_FILES_FOLDER READSTRING EXTRACT_OPTIONS FASTQ_OPTIONS; do
	CURRENT_LINE=$(( ${CURRENT_LINE}+1 ))
	RUN_ARCHIVE=`find_run_archive "${DATA_FILES_FOLDER}" "${AllRunArchives}"`
	PREFIX=${FLOWCELL_BARCODE}.${LANE}

	if [ ! -f "done/${PREFIX}.barcodes.done" ]; then
		mkdir -p ${PREFIX}
		PROGRESS=$(( ${CURRENT_LINE}*20/${NUM_LINES} ))
		rg_progress ${PROGRESS} "Extracting barcodes: ${FLOWCELL_BARCODE}; lane ${LANE} (${NumThreads} threads)"
		extract_illumina_barcodes ${RUN_ARCHIVE} ${FLOWCELL_BARCODE} ${LANE} ${READSTRING} "${EXTRACT_OPTIONS}"
		touch "done/${PREFIX}.barcodes.done";
	fi
	
	if [ ! -f "done/${PREFIX}.fastq.done" ]; then
		PROGRESS=$(( (${CURRENT_LINE}*20+10)/${NUM_LINES} ))
		rg_progress ${PROGRESS} "Creating FASTQ: ${FLOWCELL_BARCODE}; lane ${LANE} (${NumThreads} threads)"
		basecalls_to_fastq ${RUN_ARCHIVE} ${FLOWCELL_BARCODE} ${LANE} ${READSTRING} "${FASTQ_OPTIONS}"
	
		echo "# [${DEMUX_NAME}]" >> ${WD}/demultiplex_metrics.txt
		echo "# Lane=${LANE}" >> ${WD}/demultiplex_metrics.txt
		echo "# Pool=${POOL_NAME}" >> ${WD}/demultiplex_metrics.txt
		cat ${PREFIX}_metrics.csv >> ${WD}/demultiplex_metrics.txt
		echo "[${DEMUX_NAME}]" >> ${WD}/skipped_tiles.txt
		find ${PREFIX} -name "*_barcode.txt" -size 0 >> ${WD}/skipped_tiles.txt
		touch "done/${PREFIX}.fastq.done"
		rm -rf ${PREFIX}
	fi

done < ${WD}/lane_info.txt

## Read information about each merged item from 'merge_info.txt'. For each, merge FASTQ files, run
## bowtie and trimmomatic and move results to project-archive
NUM_LINES=`wc -l < ${WD}/merge_info.txt`
CURRENT_LINE=0
while IFS=$',' read MERGE_NAME FASTQ_FOLDER BASE_FASTQ_NAME UMASK; do
	CURRENT_LINE=$(( ${CURRENT_LINE}+1 ))
	PROGRESS=$(( 25+${CURRENT_LINE}*70/${NUM_LINES} ))

	if [ ! -f "done/${MERGE_NAME}.done" ]; then

		if [ "${UMASK}" ]; then
			umask -S ${UMASK}
		fi
		
		## Start new sections in result files
		echo "[${MERGE_NAME}]" >> ${WD}/trimmomatic.out
		echo "[${MERGE_NAME}]" >> ${WD}/files.out
		echo "[${MERGE_NAME}]" >> ${WD}/fragments.out
		echo "[${MERGE_NAME}]" >> ${WD}/readlength.out
		
		## Merge FASTQ files from all lanes
		if [ ! -f "done/${MERGE_NAME}.merge.done" ]; then
			rg_progress ${PROGRESS} "Consolidating FASTQ: ${MERGE_NAME}"
			merge_fastq_files fastq/${MERGE_NAME} fastq.merged/${MERGE_NAME}
			touch "done/${MERGE_NAME}.merge.done"
		fi
		
		## Align a subset with Bowtie to get an estimate of fragment sizes
		if [ ! -f "done/${MERGE_NAME}.align.done" ]; then
			rg_progress ${PROGRESS} "Bowtie2: ${MERGE_NAME} (${CURRENT_LINE} of ${NUM_LINES}; ${NumThreads} threads)"
			bowtie_align fastq.merged/${MERGE_NAME} fastq.aligned/${MERGE_NAME}
			touch "done/${MERGE_NAME}.align.done"
		fi
		
		## First Trimmomatic step is used to filter on adapter sequences
		if [ ! -f "done/${MERGE_NAME}.trim1.done" ]; then
			rg_progress ${PROGRESS} "Trimmomatic (adapter): ${MERGE_NAME} (${CURRENT_LINE} of ${NUM_LINES}; ${NumThreads} threads)"
			trimmomatic fastq.merged/${MERGE_NAME} "fastq.gz" fastq.trim.1/${MERGE_NAME} "fastq" "${TrimmomaticAdapterOptions}"
			touch "done/${MERGE_NAME}.trim1.done"
		fi
		
		## Second Trimmomatic step is used to filter on quality
		if [ ! -f "done/${MERGE_NAME}.trim2.done" ]; then
			rg_progress ${PROGRESS} "Trimmomatic (quality): ${MERGE_NAME} (${CURRENT_LINE} of ${NUM_LINES}; ${NumThreads} threads)"
			trimmomatic fastq.trim.1/${MERGE_NAME} "fastq" fastq.trim.2/${MERGE_NAME} "fastq" "${TrimmomaticQualityOptions}"
			touch "done/${MERGE_NAME}.trim2.done"
		fi
	
		## Get statitics about average remaining read lengths
		if [ ! -f "done/${MERGE_NAME}.readlength.done" ]; then
			${WD}/readlength_averager.awk < fastq.trim.2/${MERGE_NAME}_R1.fastq > fastq.trim.2/${MERGE_NAME}_readlength.txt
			${WD}/readlength_averager.awk < fastq.trim.2/${MERGE_NAME}_R2.fastq >> fastq.trim.2/${MERGE_NAME}_readlength.txt
			touch "done/${MERGE_NAME}.readlength.done"
		fi
		
		## Move results to project-archive
		rg_progress ${PROGRESS} "Archiving FASTQ: ${MERGE_NAME} (${CURRENT_LINE} of ${NUM_LINES})"
		mkdir -p ${FASTQ_FOLDER}
		rm -rf ${FASTQ_FOLDER}/*
		cat fastq.aligned/${MERGE_NAME}_fragmentsize.txt >> ${WD}/fragments.out
		cp fastq.aligned/${MERGE_NAME}_fragmentsize.txt ${FASTQ_FOLDER}/${BASE_FASTQ_NAME}_fragmentsize.txt
		cat fastq.trim.2/${MERGE_NAME}_readlength.txt >> ${WD}/readlength.out
		cp fastq.trim.2/${MERGE_NAME}_readlength.txt ${FASTQ_FOLDER}/${BASE_FASTQ_NAME}_readlength.txt
		pigz -5 -p ${NumThreads} -c fastq.trim.2/${MERGE_NAME}_R1.fastq > ${FASTQ_FOLDER}/${BASE_FASTQ_NAME}_R1.fastq.gz
		pigz -5 -p ${NumThreads} -c fastq.trim.2/${MERGE_NAME}_R2.fastq > ${FASTQ_FOLDER}/${BASE_FASTQ_NAME}_R2.fastq.gz
		ls -1 ${FASTQ_FOLDER}/*.fastq.gz >> ${WD}/files.out
		
		## Cleanup files that are no longer needed
		rm -f fastq.merged/*
		rm -f fastq.aligned/*
		rm -f fastq.trim.1/*
		rm -f fastq.trim.2/*
		
		touch "done/${MERGE_NAME}.done"	# A flag file to prevent the demux from being repeated when debugging
	fi
done < ${WD}/merge_info.txt

rg_progress 99 "Analysis completed, cleaning up..."
